@@ -2,7 +2,7 @@
 * ECC Domain Parameters
 *
 * (C) 2007 Falko Strenzke, FlexSecure GmbH
-* (C) 2008,2018 Jack Lloyd
+* (C) 2008,2018,2024 Jack Lloyd
 * (C) 2018 Tobias Niemann
 *
 * Botan is released under the Simplified BSD License (see license.txt)
@@ -16,132 +16,13 @@
 #include <botan/pem.h>
 #include <botan/reducer.h>
 #include <botan/rng.h>
+#include <botan/internal/ec_inner_data.h>
 #include <botan/internal/fmt.h>
-#include <botan/internal/point_mul.h>
 #include <botan/internal/primality.h>
 #include <vector>
 
-#if defined(BOTAN_HAS_EC_HASH_TO_CURVE)
-   #include <botan/internal/ec_h2c.h>
-#endif
-
 namespace Botan {
 
-class EC_Group_Data final {
-   public:
-      EC_Group_Data(const BigInt& p,
-                    const BigInt& a,
-                    const BigInt& b,
-                    const BigInt& g_x,
-                    const BigInt& g_y,
-                    const BigInt& order,
-                    const BigInt& cofactor,
-                    const OID& oid,
-                    EC_Group_Source source) :
-            m_curve(p, a, b),
-            m_base_point(m_curve, g_x, g_y),
-            m_g_x(g_x),
-            m_g_y(g_y),
-            m_order(order),
-            m_cofactor(cofactor),
-            m_mod_order(order),
-            m_base_mult(m_base_point, m_mod_order),
-            m_oid(oid),
-            m_p_bits(p.bits()),
-            m_order_bits(order.bits()),
-            m_a_is_minus_3(a == p - 3),
-            m_a_is_zero(a.is_zero()),
-            m_source(source) {}
-
-      bool params_match(const BigInt& p,
-                        const BigInt& a,
-                        const BigInt& b,
-                        const BigInt& g_x,
-                        const BigInt& g_y,
-                        const BigInt& order,
-                        const BigInt& cofactor) const {
-         return (this->p() == p && this->a() == a && this->b() == b && this->order() == order &&
-                 this->cofactor() == cofactor && this->g_x() == g_x && this->g_y() == g_y);
-      }
-
-      bool params_match(const EC_Group_Data& other) const {
-         return params_match(
-            other.p(), other.a(), other.b(), other.g_x(), other.g_y(), other.order(), other.cofactor());
-      }
-
-      void set_oid(const OID& oid) {
-         BOTAN_STATE_CHECK(m_oid.empty());
-         m_oid = oid;
-      }
-
-      const OID& oid() const { return m_oid; }
-
-      const BigInt& p() const { return m_curve.get_p(); }
-
-      const BigInt& a() const { return m_curve.get_a(); }
-
-      const BigInt& b() const { return m_curve.get_b(); }
-
-      const BigInt& order() const { return m_order; }
-
-      const BigInt& cofactor() const { return m_cofactor; }
-
-      const BigInt& g_x() const { return m_g_x; }
-
-      const BigInt& g_y() const { return m_g_y; }
-
-      size_t p_bits() const { return m_p_bits; }
-
-      size_t p_bytes() const { return (m_p_bits + 7) / 8; }
-
-      size_t order_bits() const { return m_order_bits; }
-
-      size_t order_bytes() const { return (m_order_bits + 7) / 8; }
-
-      const CurveGFp& curve() const { return m_curve; }
-
-      const EC_Point& base_point() const { return m_base_point; }
-
-      bool a_is_minus_3() const { return m_a_is_minus_3; }
-
-      bool a_is_zero() const { return m_a_is_zero; }
-
-      BigInt mod_order(const BigInt& x) const { return m_mod_order.reduce(x); }
-
-      BigInt square_mod_order(const BigInt& x) const { return m_mod_order.square(x); }
-
-      BigInt multiply_mod_order(const BigInt& x, const BigInt& y) const { return m_mod_order.multiply(x, y); }
-
-      BigInt multiply_mod_order(const BigInt& x, const BigInt& y, const BigInt& z) const {
-         return m_mod_order.multiply(m_mod_order.multiply(x, y), z);
-      }
-
-      BigInt inverse_mod_order(const BigInt& x) const { return inverse_mod(x, m_order); }
-
-      EC_Point blinded_base_point_multiply(const BigInt& k, RandomNumberGenerator& rng, std::vector<BigInt>& ws) const {
-         return m_base_mult.mul(k, rng, m_order, ws);
-      }
-
-      EC_Group_Source source() const { return m_source; }
-
-   private:
-      CurveGFp m_curve;
-      EC_Point m_base_point;
-
-      BigInt m_g_x;
-      BigInt m_g_y;
-      BigInt m_order;
-      BigInt m_cofactor;
-      Modular_Reducer m_mod_order;
-      EC_Point_Base_Point_Precompute m_base_mult;
-      OID m_oid;
-      size_t m_p_bits;
-      size_t m_order_bits;
-      bool m_a_is_minus_3;
-      bool m_a_is_zero;
-      EC_Group_Source m_source;
-};
-
 class EC_Group_Data_Map final {
    public:
       EC_Group_Data_Map() = default;
@@ -328,9 +209,15 @@ std::pair<std::shared_ptr<EC_Group_Data>, bool> EC_Group::BER_decode_EC_group(st
    BER_Object obj = ber.get_next_object();
 
    if(obj.type() == ASN1_Type::ObjectId) {
-      OID dom_par_oid;
-      BER_Decoder(bits).decode(dom_par_oid);
-      return std::make_pair(ec_group_data().lookup(dom_par_oid), false);
+      OID oid;
+      BER_Decoder(bits).decode(oid);
+
+      auto data = ec_group_data().lookup(oid);
+      if(!data) {
+         throw Decoding_Error(fmt("Unknown namedCurve OID '{}'", oid.to_string()));
+      }
+
+      return std::make_pair(data, false);
    }
 
    if(obj.type() == ASN1_Type::Sequence) {
@@ -528,14 +415,6 @@ const EC_Group_Data& EC_Group::data() const {
    return *m_data;
 }
 
-bool EC_Group::a_is_minus_3() const {
-   return data().a_is_minus_3();
-}
-
-bool EC_Group::a_is_zero() const {
-   return data().a_is_zero();
-}
-
 size_t EC_Group::get_p_bits() const {
    return data().p_bits();
 }
@@ -568,6 +447,10 @@ const EC_Point& EC_Group::get_base_point() const {
    return data().base_point();
 }
 
+const EC_Point& EC_Group::generator() const {
+   return data().base_point();
+}
+
 const BigInt& EC_Group::get_order() const {
    return data().order();
 }
@@ -584,30 +467,14 @@ const BigInt& EC_Group::get_cofactor() const {
    return data().cofactor();
 }
 
+bool EC_Group::has_cofactor() const {
+   return data().has_cofactor();
+}
+
 BigInt EC_Group::mod_order(const BigInt& k) const {
    return data().mod_order(k);
 }
 
-BigInt EC_Group::square_mod_order(const BigInt& x) const {
-   return data().square_mod_order(x);
-}
-
-BigInt EC_Group::cube_mod_order(const BigInt& x) const {
-   return multiply_mod_order(x, square_mod_order(x));
-}
-
-BigInt EC_Group::multiply_mod_order(const BigInt& x, const BigInt& y) const {
-   return data().multiply_mod_order(x, y);
-}
-
-BigInt EC_Group::multiply_mod_order(const BigInt& x, const BigInt& y, const BigInt& z) const {
-   return data().multiply_mod_order(x, y, z);
-}
-
-BigInt EC_Group::inverse_mod_order(const BigInt& x) const {
-   return data().inverse_mod_order(x);
-}
-
 const OID& EC_Group::get_curve_oid() const {
    return data().oid();
 }
@@ -616,102 +483,20 @@ EC_Group_Source EC_Group::source() const {
    return data().source();
 }
 
-size_t EC_Group::point_size(EC_Point_Format format) const {
-   // Hybrid and standard format are (x,y), compressed is y, +1 format byte
-   if(format == EC_Point_Format::Compressed) {
-      return (1 + get_p_bytes());
-   } else {
-      return (1 + 2 * get_p_bytes());
-   }
-}
-
-EC_Point EC_Group::OS2ECP(const uint8_t bits[], size_t len) const {
-   return Botan::OS2ECP(bits, len, data().curve());
-}
-
-EC_Point EC_Group::point(const BigInt& x, const BigInt& y) const {
-   // TODO: randomize the representation?
-   return EC_Point(data().curve(), x, y);
-}
-
-EC_Point EC_Group::point_multiply(const BigInt& x, const EC_Point& pt, const BigInt& y) const {
-   EC_Point_Multi_Point_Precompute xy_mul(get_base_point(), pt);
-   return xy_mul.multi_exp(x, y);
-}
-
-EC_Point EC_Group::blinded_base_point_multiply(const BigInt& k,
-                                               RandomNumberGenerator& rng,
-                                               std::vector<BigInt>& ws) const {
-   return data().blinded_base_point_multiply(k, rng, ws);
-}
-
-BigInt EC_Group::blinded_base_point_multiply_x(const BigInt& k,
-                                               RandomNumberGenerator& rng,
-                                               std::vector<BigInt>& ws) const {
-   const EC_Point pt = data().blinded_base_point_multiply(k, rng, ws);
-
-   if(pt.is_zero()) {
-      return BigInt::zero();
-   }
-   return pt.get_affine_x();
-}
-
-BigInt EC_Group::random_scalar(RandomNumberGenerator& rng) const {
-   return BigInt::random_integer(rng, BigInt::one(), get_order());
-}
-
-EC_Point EC_Group::blinded_var_point_multiply(const EC_Point& point,
-                                              const BigInt& k,
-                                              RandomNumberGenerator& rng,
-                                              std::vector<BigInt>& ws) const {
-   EC_Point_Var_Point_Precompute mul(point, rng, ws);
-   // We pass order*cofactor here to "correctly" handle the case where the
-   // point is on the curve but not in the prime order subgroup. This only
-   // matters for groups with cofactor > 1
-   // See https://github.com/randombit/botan/issues/3800
-   return mul.mul(k, rng, get_order() * get_cofactor(), ws);
-}
-
-EC_Point EC_Group::zero_point() const {
-   return EC_Point(data().curve());
-}
-
-EC_Point EC_Group::hash_to_curve(std::string_view hash_fn,
-                                 const uint8_t input[],
-                                 size_t input_len,
-                                 std::string_view domain,
-                                 bool random_oracle) const {
-   return this->hash_to_curve(
-      hash_fn, input, input_len, reinterpret_cast<const uint8_t*>(domain.data()), domain.size(), random_oracle);
-}
-
-EC_Point EC_Group::hash_to_curve(std::string_view hash_fn,
-                                 const uint8_t input[],
-                                 size_t input_len,
-                                 const uint8_t domain_sep[],
-                                 size_t domain_sep_len,
-                                 bool random_oracle) const {
-#if defined(BOTAN_HAS_EC_HASH_TO_CURVE)
-
-   // Only have SSWU currently
-   if(get_a().is_zero() || get_b().is_zero() || get_p() % 4 == 1) {
-      throw Not_Implemented("EC_Group::hash_to_curve not available for this curve type");
+std::vector<uint8_t> EC_Group::DER_encode() const {
+   const auto& der_named_curve = data().der_named_curve();
+   // TODO(Botan4) this can be removed because an OID will always be defined
+   if(der_named_curve.empty()) {
+      throw Encoding_Error("Cannot encode EC_Group as OID because OID not set");
    }
 
-   return hash_to_curve_sswu(*this, hash_fn, {input, input_len}, {domain_sep, domain_sep_len}, random_oracle);
-
-#else
-   BOTAN_UNUSED(hash_fn, random_oracle, input, input_len, domain_sep, domain_sep_len);
-   throw Not_Implemented("EC_Group::hash_to_curve functionality not available in this configuration");
-#endif
+   return der_named_curve;
 }
 
 std::vector<uint8_t> EC_Group::DER_encode(EC_Group_Encoding form) const {
-   std::vector<uint8_t> output;
-
-   DER_Encoder der(output);
-
    if(form == EC_Group_Encoding::Explicit) {
+      std::vector<uint8_t> output;
+      DER_Encoder der(output);
       const size_t ecpVers1 = 1;
       const OID curve_type("1.2.840.10045.1.1");  // prime field
 
@@ -731,19 +516,14 @@ std::vector<uint8_t> EC_Group::DER_encode(EC_Group_Encoding form) const {
          .encode(get_order())
          .encode(get_cofactor())
          .end_cons();
+      return output;
    } else if(form == EC_Group_Encoding::NamedCurve) {
-      const OID oid = get_curve_oid();
-      if(oid.empty()) {
-         throw Encoding_Error("Cannot encode EC_Group as OID because OID not set");
-      }
-      der.encode(oid);
+      return this->DER_encode();
    } else if(form == EC_Group_Encoding::ImplicitCA) {
-      der.encode_null();
+      return {0x00, 0x05};
    } else {
       throw Internal_Error("EC_Group::DER_encode: Unknown encoding");
    }
-
-   return output;
 }
 
 std::string EC_Group::PEM_encode() const {
@@ -777,7 +557,7 @@ bool EC_Group::verify_public_element(const EC_Point& point) const {
       return false;
    }
 
-   if(get_cofactor() > 1) {
+   if(has_cofactor()) {
       if((point * get_cofactor()).is_zero()) {
          return false;
       }
@@ -856,4 +636,30 @@ bool EC_Group::verify_group(RandomNumberGenerator& rng, bool strong) const {
    return true;
 }
 
+EC_Group::Mul2Table::Mul2Table(const EC_AffinePoint& h) : m_tbl(h._group()->make_mul2_table(h._inner())) {}
+
+EC_Group::Mul2Table::~Mul2Table() = default;
+
+std::optional<EC_AffinePoint> EC_Group::Mul2Table::mul2_vartime(const EC_Scalar& x, const EC_Scalar& y) const {
+   auto pt = m_tbl->mul2_vartime(x._inner(), y._inner());
+   if(pt) {
+      return EC_AffinePoint::_from_inner(std::move(pt));
+   } else {
+      return {};
+   }
+}
+
+bool EC_Group::Mul2Table::mul2_vartime_x_mod_order_eq(const EC_Scalar& v,
+                                                      const EC_Scalar& x,
+                                                      const EC_Scalar& y) const {
+   return m_tbl->mul2_vartime_x_mod_order_eq(v._inner(), x._inner(), y._inner());
+}
+
+bool EC_Group::Mul2Table::mul2_vartime_x_mod_order_eq(const EC_Scalar& v,
+                                                      const EC_Scalar& c,
+                                                      const EC_Scalar& x,
+                                                      const EC_Scalar& y) const {
+   return this->mul2_vartime_x_mod_order_eq(v, c * x, c * y);
+}
+
 }  // namespace Botan
