@@ -14,10 +14,14 @@
 #ifndef BOTAN_CT_UTILS_H_
 #define BOTAN_CT_UTILS_H_
 
+#include <botan/concepts.h>
 #include <botan/secmem.h>
 #include <botan/internal/bit_ops.h>
+#include <botan/internal/stl_util.h>
+
+#include <optional>
+#include <span>
 #include <type_traits>
-#include <vector>
 
 #if defined(BOTAN_HAS_VALGRIND)
    #include <valgrind/memcheck.h>
@@ -25,6 +29,9 @@
 
 namespace Botan::CT {
 
+/// @name Constant Time Check Annotation Helpers
+/// @{
+
 /**
 * Use valgrind to mark the contents of memory as being undefined.
 * Valgrind will accept operations which manipulate undefined values,
@@ -64,17 +71,178 @@ constexpr inline void unpoison(const T* p, size_t n) {
    BOTAN_UNUSED(p, n);
 }
 
-template <typename T>
-constexpr inline void unpoison(T& p) {
+/**
+ * Checks whether CT::poison() and CT::unpoison() actually have an effect.
+ *
+ * If the build is not instrumented and/or not run using an analysis tool like
+ * valgrind, the functions are no-ops and the return value is false.
+ *
+ * @returns true if CT::poison() and CT::unpoison() are effective
+ */
+inline bool poison_has_effect() {
 #if defined(BOTAN_HAS_VALGRIND)
-   if(!std::is_constant_evaluated()) {
-      VALGRIND_MAKE_MEM_DEFINED(&p, sizeof(T));
-   }
+   return RUNNING_ON_VALGRIND;
+#else
+   return false;
 #endif
-
-   BOTAN_UNUSED(p);
 }
 
+/// @}
+
+/// @name Constant Time Check Annotation Convenience overloads
+/// @{
+
+/**
+ * Poison a single integral object
+ */
+template <std::integral T>
+constexpr void poison(T& p) {
+   poison(&p, 1);
+}
+
+template <std::integral T>
+constexpr void unpoison(T& p) {
+   unpoison(&p, 1);
+}
+
+/**
+ * Poison a contiguous buffer of trivial objects (e.g. integers and such)
+ */
+template <ranges::spanable_range R>
+   requires std::is_trivially_copyable_v<std::ranges::range_value_t<R>>
+constexpr void poison(R&& r) {
+   std::span s{r};
+   poison(s.data(), s.size());
+}
+
+template <ranges::spanable_range R>
+   requires std::is_trivially_copyable_v<std::ranges::range_value_t<R>>
+constexpr void unpoison(R&& r) {
+   std::span s{r};
+   unpoison(s.data(), s.size());
+}
+
+/**
+ * Poison a class type that provides a public `_const_time_poison()` method
+ * For instance: BigInt, CT::Mask<>, FrodoMatrix, ...
+ */
+template <typename T>
+   requires requires(const T& x) { x._const_time_poison(); }
+constexpr void poison(const T& x) {
+   x._const_time_poison();
+}
+
+template <typename T>
+   requires requires(const T& x) { x._const_time_unpoison(); }
+constexpr void unpoison(const T& x) {
+   x._const_time_unpoison();
+}
+
+/**
+ * Poison an optional object if it has a value.
+ */
+template <typename T>
+   requires requires(const T& v) { ::Botan::CT::poison(v); }
+constexpr void poison(const std::optional<T>& x) {
+   if(x.has_value()) {
+      poison(x.value());
+   }
+}
+
+template <typename T>
+   requires requires(const T& v) { ::Botan::CT::unpoison(v); }
+constexpr void unpoison(const std::optional<T>& x) {
+   if(x.has_value()) {
+      unpoison(x.value());
+   }
+}
+
+/// @}
+
+/// @name Higher-level Constant Time Check Annotation Helpers
+/// @{
+
+template <typename T>
+concept poisonable = requires(const T& v) { ::Botan::CT::poison(v); };
+template <typename T>
+concept unpoisonable = requires(const T& v) { ::Botan::CT::unpoison(v); };
+
+/**
+ * Poison a range of objects by calling `poison` on each element.
+ */
+template <std::ranges::range R>
+   requires poisonable<std::ranges::range_value_t<R>>
+constexpr void poison_range(R&& r) {
+   for(const auto& v : r) {
+      poison(v);
+   }
+}
+
+template <std::ranges::range R>
+   requires unpoisonable<std::ranges::range_value_t<R>>
+constexpr void unpoison_range(R&& r) {
+   for(const auto& v : r) {
+      unpoison(v);
+   }
+}
+
+/**
+ * Poisons an arbitrary number of values in a single call.
+ * Mostly syntactic sugar to save clutter (i.e. lines-of-code).
+ */
+template <poisonable... Ts>
+   requires(sizeof...(Ts) > 0)
+constexpr void poison_all(Ts&&... ts) {
+   (poison(ts), ...);
+}
+
+template <unpoisonable... Ts>
+   requires(sizeof...(Ts) > 0)
+constexpr void unpoison_all(Ts&&... ts) {
+   (unpoison(ts), ...);
+}
+
+/**
+ * Poisons an arbitrary number of poisonable values, and unpoisons them when the
+ * returned object runs out-of-scope
+ *
+ * Use this when you want to poison a value that remains valid longer than the
+ * scope you are currently in. For instance, a private key structure that is a
+ * member of a Signature_Operation object, that may be used for multiple
+ * signatures.
+ */
+template <typename... Ts>
+   requires(sizeof...(Ts) > 0) && (poisonable<Ts> && ...) && (unpoisonable<Ts> && ...)
+[[nodiscard]] constexpr auto scoped_poison(const Ts&... xs) {
+   auto scope = scoped_cleanup([&] { unpoison_all(xs...); });
+   poison_all(xs...);
+   return scope;
+}
+
+/**
+ * Poisons an r-value @p v and forwards it as the return value.
+ */
+template <poisonable T>
+[[nodiscard]] decltype(auto) driveby_poison(T&& v)
+   requires(std::is_rvalue_reference_v<decltype(v)>)
+{
+   poison(v);
+   return std::forward<T>(v);
+}
+
+/**
+ * Unpoisons an r-value @p v and forwards it as the return value.
+ */
+template <unpoisonable T>
+[[nodiscard]] decltype(auto) driveby_unpoison(T&& v)
+   requires(std::is_rvalue_reference_v<decltype(v)>)
+{
+   unpoison(v);
+   return std::forward<T>(v);
+}
+
+/// @}
+
 /**
 * This function returns its argument, but (if called in a non-constexpr context)
 * attempts to prevent the compiler from reasoning about the value or the possible
@@ -98,7 +266,7 @@ constexpr inline T value_barrier(T x) {
       * however the current approach seems sufficient with current compilers,
       * and is minimally damaging with regards to degrading code generation.
       */
-#if defined(BOTAN_USE_GCC_INLINE_ASM)
+#if defined(BOTAN_USE_GCC_INLINE_ASM) && !defined(BOTAN_HAS_SANITIZER_MEMORY)
       asm("" : "+r"(x) : /* no input */);
 #endif
       return x;
@@ -147,6 +315,8 @@ class Choice final {
 
       constexpr Choice operator!=(const Choice& other) const { return Choice(value() ^ other.value()); }
 
+      constexpr Choice operator==(const Choice& other) const { return !(*this != other); }
+
       /**
       * Unsafe conversion to bool
       *
@@ -173,6 +343,12 @@ class Choice final {
       uint32_t m_value;
 };
 
+/**
+* A concept for a type which is conditionally assignable
+*/
+template <typename T>
+concept ct_conditional_assignable = requires(T lhs, const T& rhs, Choice c) { lhs.conditional_assign(c, rhs); };
+
 /**
 * A Mask type used for constant-time operations. A Mask<T> always has value
 * either |0| (all bits cleared) or |1| (all bits set). All operations in a Mask<T>
@@ -230,6 +406,14 @@ class Mask final {
       */
       static constexpr Mask<T> expand_top_bit(T v) { return Mask<T>(Botan::expand_top_bit<T>(value_barrier<T>(v))); }
 
+      /**
+       * Return a Mask<T> which is set if the given @p bit of @p v is set.
+       * @p bit must be from 0 (LSB) to (sizeof(T) * 8 - 1) (MSB).
+       */
+      static constexpr Mask<T> expand_bit(T v, size_t bit) {
+         return CT::Mask<T>::expand_top_bit(v << (sizeof(v) * 8 - 1 - bit));
+      }
+
       /**
       * Return a Mask<T> which is set if m is set
       */
@@ -371,8 +555,9 @@ class Mask final {
       * cleared (resp)
       */
       constexpr void select_n(T output[], const T x[], const T y[], size_t len) const {
+         const T mask = value();
          for(size_t i = 0; i != len; ++i) {
-            output[i] = this->select(x[i], y[i]);
+            output[i] = choose(mask, x[i], y[i]);
          }
       }
 
@@ -416,12 +601,100 @@ class Mask final {
       */
       constexpr T value() const { return value_barrier<T>(m_mask); }
 
+      constexpr void _const_time_poison() const { CT::poison(m_mask); }
+
+      constexpr void _const_time_unpoison() const { CT::unpoison(m_mask); }
+
    private:
       constexpr Mask(T m) : m_mask(m) {}
 
       T m_mask;
 };
 
+/**
+* A CT::Option<T> is either a valid T, or not
+*
+* To maintain constant time behavior a value must always be stored.
+* A CT::Choice tracks if the value is valid or not. It is not possible
+* to access the inner value if the Choice is unset.
+*/
+template <typename T>
+class Option {
+   public:
+      /// Construct an Option which contains the specified value, and is set or not
+      constexpr Option(T v, Choice valid) : m_has_value(valid), m_value(std::move(v)) {}
+
+      /// Construct a set option with the provided value
+      constexpr Option(T v) : Option(std::move(v), Choice::yes()) {}
+
+      /// Construct an unset option with a default inner value
+      constexpr Option()
+         requires std::default_initializable<T>
+            : Option(T(), Choice::no()) {}
+
+      /// Return true if this Option contains a value
+      constexpr Choice has_value() const { return m_has_value; }
+
+      /**
+      * Apply a function to the inner value and return a new Option
+      * which contains that value. This is constant time only if @p f is.
+      *
+      * @note The function will always be called, even if the Option is None. It
+      *       must be prepared to handle any possible state of T.
+      */
+      template <std::invocable<const T&> F>
+      constexpr auto transform(F f) const -> Option<std::remove_cvref_t<std::invoke_result_t<F, const T&>>> {
+         return {f(m_value), m_has_value};
+      }
+
+      /// Either returns the value or throws an exception
+      constexpr const T& value() const {
+         BOTAN_STATE_CHECK(m_has_value.as_bool());
+         return m_value;
+      }
+
+      /// Returns either the inner value or the alternative, in constant time
+      ///
+      /// This variant is used for types which explicitly define a function
+      /// conditional_assign which takes a CT::Choice as the conditional.
+      constexpr T value_or(T other) const
+         requires ct_conditional_assignable<T>
+      {
+         other.conditional_assign(m_has_value, m_value);
+         return other;
+      }
+
+      /// Returns either the inner value or the alternative, in constant time
+      ///
+      /// This variant is used for integer types where CT::Mask can perform
+      /// a constant time selection
+      constexpr T value_or(T other) const
+         requires std::unsigned_integral<T>
+      {
+         auto mask = CT::Mask<T>::from_choice(m_has_value);
+         return mask.select(m_value, other);
+      }
+
+      /// Convert this Option into a std::optional
+      ///
+      /// This is not constant time, leaking if the Option had a
+      /// value or not
+      constexpr std::optional<T> as_optional_vartime() const {
+         if(m_has_value.as_bool()) {
+            return {m_value};
+         } else {
+            return {};
+         }
+      }
+
+      /// Return a new CT::Option that is set if @p also is set as well
+      constexpr CT::Option<T> operator&&(CT::Choice also) { return CT::Option<T>(m_value, m_has_value && also); }
+
+   private:
+      Choice m_has_value;
+      T m_value;
+};
+
 template <typename T>
 constexpr inline Mask<T> conditional_copy_mem(Mask<T> mask, T* to, const T* from0, const T* from1, size_t elems) {
    mask.select_n(to, from0, from1, elems);
@@ -513,28 +786,33 @@ constexpr inline CT::Mask<T> is_not_equal(const T x[], const T y[], size_t len)
 }
 
 /**
-* If bad_input is unset, return input[offset:input_length] copied to new
-* buffer. If bad_input is set, return an empty vector. In all cases, the capacity
-* of the vector is equal to input_length
+* Constant time conditional copy out with offset
 *
-* This function attempts to avoid leaking the following:
-*  - if bad_input was set or not
+* If accept is set and offset <= input_length, sets output[0..] to
+* input[offset:input_length] and returns input_length - offset. The
+* remaining bytes of output are zeroized.
+*
+* Otherwise, output is zeroized, and returns an empty Ct::Option
+*
+* The input and output spans may not overlap, and output must be at
+* least as large as input.
+*
+* This function attempts to avoid leaking the following to side channels
+*  - if accept was set or not
 *  - the value of offset
-*  - the values in input[]
+*  - the value of input
 *
-* This function leaks the value of input_length
+* This function leaks the length of the input
 */
 BOTAN_TEST_API
-secure_vector<uint8_t> copy_output(CT::Mask<uint8_t> bad_input,
-                                   const uint8_t input[],
-                                   size_t input_length,
-                                   size_t offset);
+CT::Option<size_t> copy_output(CT::Choice accept,
+                               std::span<uint8_t> output,
+                               std::span<const uint8_t> input,
+                               size_t offset);
 
-secure_vector<uint8_t> strip_leading_zeros(const uint8_t in[], size_t length);
+size_t count_leading_zero_bytes(std::span<const uint8_t> input);
 
-inline secure_vector<uint8_t> strip_leading_zeros(const secure_vector<uint8_t>& in) {
-   return strip_leading_zeros(in.data(), in.size());
-}
+secure_vector<uint8_t> strip_leading_zeros(std::span<const uint8_t> input);
 
 }  // namespace Botan::CT
 
