@@ -56,6 +56,25 @@ uint64_t BOTAN_TEST_API get_cpu_cycle_counter();
 
 size_t BOTAN_TEST_API get_cpu_available();
 
+/**
+* Return true if get_auxval is implemented on this system
+*/
+bool has_auxval();
+
+/**
+* If get_auxval is supported, returns the relevant value for AT_HWCAP
+*
+* If get_auxval is not supported on this system, arbitrarily returns 0
+*/
+unsigned long auxval_hwcap();
+
+/**
+* If get_auxval is supported, returns the relevant value for AT_HWCAP2
+*
+* If get_auxval is not supported on this system, arbitrarily returns 0
+*/
+unsigned long auxval_hwcap2();
+
 /**
 * Return the ELF auxiliary vector cooresponding to the given ID.
 * This only makes sense on Unix-like systems and is currently
